#ifndef algorithmComposite_H
#define algorithmComposite_H

#include "Event/algorithmVI.h"
#include "Event/serverVI.h"

//---------------------------------------------
//! An algorithm composed by several algorithms
/*! 
\e algorithmComposite is an algorithmVI.<br>
\e algorithmComposite is a composite of algorithms. <br>
<ul>
	<li> The \c run() method runs all the \c run() methods of all the algorithms. <br>
	<li> It uses the serverVI <algorithmVI> class to create the composite.
</ul>
*/
//----------------------------------------------
//             J.A Hernando, Santa Cruz 02/29/00
//----------------------------------------------

//###########################
class algorithmComposite : public algorithmVI , public serverVI<algorithmVI>
//###########################
{
public:


	//! default constructor
	algorithmComposite(){}
	//! default destructor
	virtual ~algorithmComposite() {};  

	//! initializes all the algorithms
	virtual void initialize()  {serverVI<algorithmVI>::command("initialize");}
	//! executes all the algorithms
	virtual void execute()     {serverVI<algorithmVI>::command("execute");}
	//! finalizes all the algorithms
	virtual void finalize()    {serverVI<algorithmVI>::command("finalize");}
	//! runs all the algorithms
	virtual void run()         {serverVI<algorithmVI>::command("run");}
};
#endif
