#ifndef algorithmImport_h
#define algorithmImport_h

#include "Event/algorithmCommand.h"

//! template class to convert a void member function of a class into an algorithmCommand
/*!
\e algorihtmImport is an algorithmVI. <br>
\e algorithmImport takes as arguments: a template class and a pointer to a
void member function of this class.
<ul>
	<li> the algorithm imported runs the member function of the template class.
	<li> this template class is very useful to create algorithms from already
	existing classes which void methods without arguments could be in fact algorithms.
	<li> the created algorithm should be incorporated into processManager.
</ul>
*/
//###############################################
template<class T> class algorithmImport : public algorithmCommand
//###############################################
{
	typedef void (T::* ptr_mem) ();

public:
	
	//! constructor - set the template and the member function to run
	algorithmImport(T* t, ptr_mem run):m_t(t),m_run(run) {}
	//! default destructor
	virtual ~algorithmImport() {}

	//! execute as algorithm the member function of the template class
	virtual void execute() {(m_t->* m_run)();}

private:

	//! pointer to the template
	T* m_t;
	//! pointer to the template member function
	ptr_mem m_run;
};
#endif