#ifndef dataIOROOTServer_h
#define dataIOROOTServer_h

#include "Event/IOfileServer.h"

#include "centella/userIOfiles.h"

//-------------------------------------------------
//! It stores the ROOT trees Servers and histograms
/*!
\e dataIOROOTServer is an implementation of permanent data Server for ROOT.<br>
\e dataIOROOTServer has the services of the IOServer.<br>
\e dataIOROOTServer has defined: rTreeInServer, rTreeOutServer, rHistoOutServer. For
the input tree, output tree and output histogram main file. 
The \b user should typedef to his/her implementation classes in userIOfiles.
<ul>
	<li> The IOfileVI operations are handle via the IOServer.
	<li> The ROOT permanent data is the core (or main branch) of the input/output
	ROOT trees. Some of the converters or algorithms maybe need access to the
	core. The core is provided via treeIn()->core() method of the rTreeBase class.
	<li> The user should typedef (see header file) their implementation class of the input and
	ouput ROOT trees (rTreeInServer and rTreeOutServer). 
	<li> The user should typedef (see header file) their implementation class of rHistoOutServer.
	<li> If not output files (either tree o histo) are going to be used, the user
	should typedef to the IOfileNull class (a system defined IOfileVI null class).
	</ul>
	Notes:
	<ul>
	<li> a rTreeInServer (rTreeOutServer) is typedef to a user defined class that should inherits from:
	<ul>
		<li> the template class rTreeBase, where the template argument is
		the core (main branch) of the ROOT tree.
		<li> from a converterServer that will convert the converterVI s into
		algLoad and algSave algorithms.
	</ul>
	<li> a rHistoOutServer is typedef to a user defined class that should inherits from a
	rHistoServer class. The user should:
	<ul>
		<li> define a collection of rHistoFolder s.
		<li> add then into the server (using the method \c addHistoFolder() of
		rHistoFolderServer), implementing the pure virtual method \c defineHistoFolders().
	</ul>
</ul>
*/
//------------------------------------
// JA Hernando, Santa Cruz, CA, 02/15/00
//------------------------------------

//####################################
class dataIOROOTServer : public IOfileServer
//####################################
{
	friend class dataManager;

public:

	//! constructor
	dataIOROOTServer(); 
	//! destructor
	~dataIOROOTServer();  

	//! returns a pointer to the Input root-tree Server
	rTreeInServer*    treeIn() const;
	//! returns a pointer to the Output root-tree Server
	rTreeOutServer*   treeOut() const;
	//! returns a pointer to the histo Server
	rHistoOutServer*     histo() const;   

protected:

	//! define the input and output IOfileVI
	virtual void defineIOfiles();

private:

	//! pointer to the input root-tree Server
	rTreeInServer*   m_TreeIn;
	//! pointer to the output root-tree Server
	rTreeOutServer*  m_TreeOut;
	//! pointer to the histo Server
	rHistoOutServer*    m_Histo;

};
#endif

