#ifndef eventVI_h
#define eventVI_h

#include "Event/algorithmVI.h"
#include "Event/algorithmTree.h"

//--------------------------------
//! eventVI : bare class to define an algorithm EVENT
/*! 
\e eventVI is an algorithmVI.<br>
\e eventVI is the main algorithm to execute the event tasks: generation, reconstruction and
analysis.
<ul>
	<li> It reuses an algorithmTree.
	<li> An event algorithm reflect the "physical" process of an \b event. An event
	is the process of an interacion of a particle with the detector and how the physicits
	retrieve the initial information from the detector response. An event has
	three separated task: \b generation (from the interaction of the particle to
	the response of the detector) this algorithm creates the Raw data;
	\b reconstruction (from the raw data to reconstructed data, data which is still not physical); 
	and	\b analysis (from reconstructed data to physics variables). 
	Example: tracker device: 1) generation: from the interacion of the particle to the list
	of hits, 2) reconstruction: from the list of hits to reconstructed tracks; 3) analysis
	from the list of track to the initial direction of the incident particle.
	<li> An event algorithm executes three algorithms (also methods): \c generate(), 
	\c reconstruct(), and \c analyze(). Each one is an algorithmComposite, so it can be
	composed of several algorithms.
	<li> This class uses an algorithmTree and renames
	 the initalize, execute and finalize methods into generate, 
	 reconstruct and analyze methods.
</ul>
*/
//----------------------------
// JA Hernando, Santa Cruz, CA, 05/15/00
//----------------------------

//############################
class eventVI : public algorithmTree
//############################
{
public:


	//! constructor, construct the pointers to the algorithmComposites
	eventVI() {};
	//! destructor, destroies the pointers to the algorithmComposites
	virtual ~eventVI() {};

	//! overwrite the virtuam method run() that now executes the generate(), reconstruct(), and analysis() methods;
	/*! this methods are in fact algorithmComposites from algorithmTree initialize(),
	execute() and finalize() */
	virtual void run();

	//! generate the process running an algorithmComposite 
	virtual void generate()    {initialize();}
	//! reconstruct the process running an algorithmComposite
	virtual void reconstruct() {execute();}
	//! reconstruct the porcess running an algorithmComposite
	virtual void analyze()     {finalize();}

	//! execute the methods via the command
	virtual void command(std::string com);

protected:

	//! pointer to generate algorithmComposite
	algorithmComposite* genAlg() const {return iniAlg();}
	//! pointer to the reconstruct algorithmComposite
	algorithmComposite* recAlg() const {return exeAlg();}
	//! pointer to the analyze algorithmComposite
	algorithmComposite* anaAlg() const {return finAlg();} 

};
#endif