
#include "Event/messageManager.h"
#include "Event/optionManager.h"

messageManager* messageManager::m_instance = 0;

//###########################################
messageManager::messageManager() 
//###########################################
{
	if (m_instance !=0 ) return;

	m_server = new serverVI<messageVI>();

	m_definedFile = false;
	m_fileName = "EMPTY";
	m_level = "RELEASE";
	m_instance = this;

	/*! gets service to messageManager */
	setName("messageManager");
	/*! make a favor: optionManager is constructed before messageManager.*/
	addToServer("optionManager",optionManager::instance());
}
//###########################################
void messageManager::open(std::string fileName)
//###########################################
{
	if (m_definedFile) m_file.close();
	m_definedFile = true;
	m_file.open(fileName.c_str());
}
//###########################################
void messageManager::close() 
//###########################################
{
	m_definedFile = false;
	m_file.close();
}
//##################################
void messageManager::setOption(std::string varname, std::string content)
//##################################
{
	optionVI::setOption(varname,content);
	if (varname == "outFile") open(content);
	if (varname == "level") setLevel(content);
}
//###########################################
messageManager::~messageManager()
//###########################################
{
	if (m_definedFile) close();
} 
//###########################################
void messageManager::addToServer(std::string name, messageVI* mp) 
//###########################################
{
	if (!m_server->search(name)) m_server->add(name,mp);
	else message(" object already in server "+name);
}
//###########################################
std::ostream& messageManager::out() const
//###########################################
{
	if (m_instance == 0) return std::cout;
	if (m_instance->m_definedFile) return m_instance->m_file;
	else return std::cout;
}
//###########################################
void messageManager::message(std::string comment, std::string ilevel) const
//###########################################
{
	if (acceptLevel(ilevel)) out() << "* " << comment << " * \n";
}
//###########################################
void messageManager::message(std::string comment, double val, std::string ilevel) const 
//###########################################
{
	if (acceptLevel(ilevel)) out() << "* " << comment << " " << val <<" * \n";
}

//###########################################
bool messageManager::acceptLevel(std::string slevel) const 
//###########################################
{
	return (messageBase::getLevel(slevel) <= getLevel()); 
}

//########################################
void messageManager::defineOption()
//########################################
{
	//! this two options are also an operation
	optionVI::defineOption("outFile",&m_fileName);
	optionVI::defineOption("level",&m_level);
}
//########################################
messageVI* messageManager::getMessage(std::string name) const
//########################################
{
	if (!m_server->search(name)) {
		message( " no seach class in Server : "+name);
		return 0;
	} 
	return m_server->get(name);
}
//########################################
void messageManager::writeOutServer() const
//########################################
{
	for (int i=0; i < m_server->size(); i++) m_server->get(i)->writeOut();
}
//########################################
void messageManager::writeOut(std::string name) const
//########################################
{
	messageVI* mes = getMessage(name);
	if (mes->acceptLevel()) mes->writeOut();
}
//########################################
void messageManager::writeOut() const
//########################################
{
	std::ostream& out = messageManager::instance()->out();
	if (m_server->size() == 0) return;
	messageVI::writeOut();
	out << " -- objects defined in the message Server -- " <<"\n";
	out << m_server->nameList();
}
