
#ifndef messageMANAGER_H
#define messageMANAGER_H 1 

#include <string>
#include <stdlib.h>
#include <iostream>
#include <fstream>

#include "Event/serviceI.h"
//-------------------------------------------------
//! Manager to handle the message of the classes and define an ostream output
/*! 
<ul>
	<li> It defines an ostream& output (method \c out()) where to send the messages. The
	output can be a file or the screen (std::cout). The method \c open() define and open 
	a file to send the messages to.
	<li> A collection of message() methods allows a external class to send a message
	(either a string or a string + a double value) to the ostream output. The messageManager
	has a enumeration LEVEL (from messageBase) that defines which are the messages that
	the messageManager will process out, via the \c acceptLevel() method. The message()
	methods are inherits and overwriten from messageVI.
	<li> It has a protected server with the list of classes that can be served with
	messages (that inherit from messageVI). the method \c writeOut(name) executes the
	\c writeOut() method of the object with name stored in the server.
</ul>
*/
//----------------------------------------------
//             J.A Hernando, Santa Cruz 02/29/00
//----------------------------------------------

//########################################
class messageManager : public serviceI
//########################################
{
	friend class messageVI;

public:

	//! default constructor
	messageManager();
	//! default constructor
	virtual ~messageManager();

	//! Singleton
	static messageManager* instance() {return m_instance;}

	//! ostream of the output
	std::ostream& out() const;

	//! open the output fileName file for messages
	void open(std::string fileName);
	//! close the output file for messages
	void close();

	//! send a string message with level (messageVI)
	void message(std::string message, std::string ilevel = "RELEASE") const ;
	//! send a string + double message with level (messageVI)
	void message(std::string message, double val, std::string ilevel = "RELEASE") const;

	//! returns true if the string level is smaller that the messageManager level
	bool acceptLevel(std::string slevel) const;
		
	//! add an object, with a name into the server
	void addToServer(std::string name, messageVI* mp);

	//! execute writeOut for all the objects in the server
	void writeOutServer() const;

	//! execute the writeOut() method for the object with name;
	void writeOut(std::string name) const;

	//! writeOut the information of the messageManager only
	void writeOut() const;

protected:

	//! returns the messageVI of the object with name
	messageVI* getMessage(std::string name) const;

	//! define the options of this class
	virtual void defineOption();
	

	//! overwrites virtual method to define option
	/*! It open the file or sets the level*/
	virtual void setOption(std::string var, std::string content); // overwriten

private:

	//! Singleton: pointer to itself
	static messageManager* m_instance;

	//! server with the classes derived from messageVI
	serverVI<messageVI>* m_server;

	//! level of the message
	std::string m_level;
	//! name of the output file
	std::string m_fileName;

	//! bool is the file has been defined
	bool m_definedFile;

	//! the ostream of the output
	std::ofstream m_file;
};

#endif
