
#ifndef rHistoFileServer_h
#define rHistoFileServer_h

#include "Event/rTFileBase.h"
#include "Event/rHistoFolderServer.h"

//----------------------------------------
//! Base class to create a ROOT file with histograms and ntuples.
/*! 
\e HistoFileServer shares the Input/Output interface defined in IOfileVI via rTFileBase. <br>
\e HistoFileServer has an rHistoFolderServer that contains a collection
of rHistoFolder located in the same root TFile file.
<ul>
	<li> The <b> user </b> can define an output ROOT file with histogram and
	ntuples, using this class as a base class. <br>
	<li> In the derived class the user should create and add his rHistoFolder s into
	the system. For that he/she should write the method \c defineHistoFolders()
	see rHistoFolderServer for more information.
	<li> HistoFileServer modifies the \c open() of rTFileBase to set the same TFile into
	all the rHistoFolders of the rHistoFolderServer. In this way all the rHistoFolders
	are located in the same ROOT TFile.
</ul>
*/
//----------------------------------------
//   JA Hernando, Santa Cruz, CA, 05/15/00
//----------------------------------------

//######################
class rHistoFileServer : public rHistoFolderServer, public rTFileBase
//######################
{
protected:

	//! generic constructor. 
	/*! A list of User defined "rHistoFolder" are created and stored*/
	rHistoFileServer() {};
	//! destructor
	virtual ~rHistoFileServer() {};

	//! the user should define the folders
	virtual void defineHistoFolders() = 0 ;

	//! Open a root-file with a given name
	/*! overwriten to set the TFile into the rHistoFolders*/
	virtual void open(); 
};

#endif

